#! python

''' River Crossing'''

import sys, time
from copy import copy
from queue import Queue
from itertools import combinations_with_replacement



BOAT_SIZE = 2
N_OF_COUPLE = 3


class Person:
    '''People crossing river:
    Couple No = {1,2,3}
    isBoy = True or False
    '''
    
    def __init__(self, couple_no, gender):
        self.no = couple_no
        self.isboy = gender=='boy'

    def __hash__(self):
        return self.no * 2 + self.isboy

    def is_girl(self):
        return not self.isboy

    def is_paired(self, ls):
        '''if there is a partner for the self in ls'''
        return any(self.no == other.no and self.isboy ^ other.isboy for other in ls)

    def __str__(self):
        return ('B' if self.isboy else 'G')  + str(self.no)




class Place (frozenset):
    '''Representing river sides and boat.
    The state of the place is determined by a conventional hash function.'''

    def __init__(self, s=[]):
       frozenset.__init__(self,s)

    def __eq__(self, other):
        return  hash(self) == hash(other)

    def all_girl(self):
        return all( x.is_girl() for x in self )

    def all_girls_paired(self):
        ls_g = [ x for x in self if x.is_girl() ]
        return all( x.is_paired(self) for x in ls_g)

    def is_safe(self):
        return self.all_girl() or self.all_girls_paired()
   
    def __str__(self):
       return '{' + ', '.join( str(x) for x in self ) + '}'

    def __add__(self, other):
        return Place(frozenset.__or__(self, other))

    def __sub__(self, other):
        return Place(frozenset.__sub__(self, other))

    def __hash__(self):
        return frozenset.__hash__(self)




class River:
    '''River consists of the left side, the right side, a boat, and the boat position'''
    
    def __init__(self, lhs, rhs=Place(), boat=Place(), boat_is_left=True):
        self.lhs = lhs
        self.rhs = rhs
        self.boat = boat
        self.boat_is_left=boat_is_left

    def __str__(self):
        return str(self.lhs) +  \
                '   ' * (not self.boat_is_left) + \
                str(self.boat) + \
               '   ' * self.boat_is_left + \
               str(self.rhs)

    def is_safe(self):
        return self.lhs.is_safe() and self.rhs.is_safe() and self.boat.is_safe() and \
               ( (self.lhs + self.boat).is_safe()  if self.boat_is_left else (self.rhs + self.boat).is_safe() )

    def __eq__(self, other):
        return hash(self)==hash(other)

    def is_finish(self):
        return not self.lhs

    def __hash__(self):
        return hash( ( self.boat_is_left, self.boat, self.lhs, self.rhs) )

    def proceed(self, visited):
        if self.boat_is_left:
            tmp = self.lhs + self.boat
            s= { River(tmp - boat, self.rhs, boat, not self.boat_is_left) for boat in onboat(tmp, BOAT_SIZE) }
        else:
            tmp = self.rhs + self.boat
            s= { River(self.lhs , tmp - boat , boat, not self.boat_is_left) for boat in onboat(tmp, BOAT_SIZE) }

        return { r for r in s if r.is_safe() and r not in visited }




def onboat(p, n):
    ''' select upto n from p '''
    return  ( Place(c) for c in combinations_with_replacement(p, n) )


def cons(x, ls):
    ls1=copy(ls)
    ls1.append(x)
    return ls1


def print_path(ls):
    '''print path'''
    print('\nResult:\nStep\t{Left Side} {Boat} {Right Side}')
    for i, x in enumerate(ls):
        print ('[{0:2d}]\t{1}'.format(i,x))


def river_cross(p0):
    visited={p0}
    q = Queue()
    q.put([p0])
    while not q.empty():
        #print(q.qsize())
        path = q.get()
        now = path[-1]
        for _next in now.proceed(visited):
            path1= cons(_next, path)
            if _next.is_finish():
                print_path(path1)
                return
            else:
                q.put(path1)
                visited.add(_next)




if __name__=='__main__':
    if len(sys.argv)==3 and sys.argv[1].isdigit() and sys.argv[2].isdigit():
        N_OF_COUPLE = int(sys.argv[1])
        BOAT_SIZE = int(sys.argv[2])

    t0=time.time()
    river_cross(  River( Place(                                                  \
       [ Person(i, gdr) for i in range(N_OF_COUPLE) for gdr in ('boy', 'girl') ]
    )))
    t1=time.time()
    print('\n{:3.3f} sec'.format(t1-t0))
